@@ -1,7 +1,7 @@
 /* -----------------------------------------------------------------------------
 Software License for The Fraunhofer FDK AAC Codec Library for Android
 
-© Copyright  1995 - 2018 Fraunhofer-Gesellschaft zur Förderung der angewandten
+© Copyright  1995 - 2019 Fraunhofer-Gesellschaft zur Förderung der angewandten
 Forschung e.V. All rights reserved.
 
  1.    INTRODUCTION
@@ -446,8 +446,26 @@ void lppTransposer(
                                 pSettings->nCols) +
                      lowBandShift);
     }
-    dynamicScale = fixMax(
-        0, dynamicScale - 1); /* one additional bit headroom to prevent -1.0 */
+
+    if (dynamicScale == 0) {
+      /* In this special case the available headroom bits as well as
+         ovLowBandShift and lowBandShift are zero. The spectrum is limited to
+         prevent -1.0, so negative values for dynamicScale can be avoided. */
+      for (i = 0; i < (LPC_ORDER + pSettings->overlap + pSettings->nCols);
+           i++) {
+        lowBandReal[i] = fixMax(lowBandReal[i], (FIXP_DBL)0x80000001);
+      }
+      if (!useLP) {
+        for (i = 0; i < (LPC_ORDER + pSettings->overlap + pSettings->nCols);
+             i++) {
+          lowBandImag[i] = fixMax(lowBandImag[i], (FIXP_DBL)0x80000001);
+        }
+      }
+    } else {
+      dynamicScale =
+          fixMax(0, dynamicScale -
+                        1); /* one additional bit headroom to prevent -1.0 */
+    }
 
     /*
       Scale temporal QMF buffer.
@@ -996,8 +1014,8 @@ void lppTransposerHBE(
                               pSettings->nCols) +
                    lowBandShift);
 
-    dynamicScale = fixMax(
-        0, dynamicScale - 1); /* one additional bit headroom to prevent -1.0 */
+    dynamicScale =
+        dynamicScale - 1; /* one additional bit headroom to prevent -1.0 */
 
     /*
     Scale temporal QMF buffer.
@@ -1176,6 +1194,9 @@ void lppTransposerHBE(
     } else { /* bw <= 0 */
 
       int descale = fixMin(DFRACT_BITS - 1, (LPC_SCALE_FACTOR + dynamicScale));
+      dynamicScale +=
+          1; /* prevent negativ scale factor due to 'one additional bit
+                headroom' */
 
       for (i = startSample; i < stopSample; i++) {
         FIXP_DBL accu1, accu2;
@@ -1192,9 +1213,9 @@ void lppTransposerHBE(
                 dynamicScale;
 
         qmfBufferReal[i][loBand] =
-            (lowBandReal[LPC_ORDER + i] >> descale) + (accu1 << 1);
+            (lowBandReal[LPC_ORDER + i] >> descale) + (accu1 << (1 + 1));
         qmfBufferImag[i][loBand] =
-            (lowBandImag[LPC_ORDER + i] >> descale) + (accu2 << 1);
+            (lowBandImag[LPC_ORDER + i] >> descale) + (accu2 << (1 + 1));
       }
     } /* bw <= 0 */
 
